!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
!		Torus Bifurcation in an Electronic Oscillator
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 

 SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP) 
!---------- ---- 

 IMPLICIT NONE
 INTEGER, INTENT(IN) :: NDIM, ICP(*), IJAC
 DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
 DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
 DOUBLE PRECISION, INTENT(INOUT) :: DFDU(NDIM,NDIM), DFDP(NDIM,*)

 DOUBLE PRECISION nu, beta, gamma, r, a3, b3, x, y, z

   nu    = PAR(1) 
   beta  = PAR(2) 
   gamma = PAR(3) 
   r     = PAR(4) 
   a3    = PAR(5) 
   b3    = PAR(6) 

   x = U(1)
   y = U(2)
   z = U(3)

   F(1) = ( -(beta+nu)*x + beta*y - a3*x**3 + b3*(y-x)**3 )/r
   F(2) = beta*x - (beta+gamma)*y - z - b3*(y-x)**3
   F(3) = y

  IF(IJAC.EQ.0)RETURN

   DFDU(1,1) = ( -(beta+nu) -3*a3*x**2 - 3*b3*(y-x)**2  )/r
   DFDU(1,2) = ( beta + 3*b3*(y-x)**2 )/r
   DFDU(1,3) = 0

   DFDU(2,1) = beta + 3*b3*(y-x)**2
   DFDU(2,2) =-(beta+gamma) - 3*b3*(y-x)**2
   DFDU(2,3) =-1

   DFDU(3,1) = 0
   DFDU(3,2) = 1
   DFDU(3,3) = 0

 IF(IJAC.EQ.1)RETURN 

! *Parameter derivatives
   DFDP(1,1) =-x/r
   DFDP(2,1) = 0
   DFDP(3,1) = 0

   DFDP(1,2) = ( -x + y )/r
   DFDP(2,2) = x-y
   DFDP(3,2) = 0

   DFDP(1,3) = 0
   DFDP(2,3) =-y
   DFDP(3,3) = 0

   DFDP(1,4) =-F(1)/r
   DFDP(2,4) = 0
   DFDP(3,4) = 0

   DFDP(1,5) = x**3/r
   DFDP(2,5) = 0
   DFDP(3,5) = 0

   DFDP(1,6) = (y-x)**3 / r
   DFDP(2,6) =-(y-x)**3
   DFDP(3,6) = 0

 END SUBROUTINE FUNC
!---------------------------------------------------------------------- 

 SUBROUTINE STPNT(NDIM,U,PAR,T)
!---------- ----- 

 IMPLICIT NONE
 INTEGER, INTENT(IN) :: NDIM
 DOUBLE PRECISION, INTENT(INOUT) :: U(NDIM),PAR(*)
 DOUBLE PRECISION, INTENT(IN) :: T

 DOUBLE PRECISION nu, beta, gamma, r, a3, b3, x, y, z

   nu    = -0.9d0
   beta  =  0.5d0
   gamma = -0.6d0
   r     =  0.6d0
   a3    = 0.328578d0
   b3    = 0.933578d0

   PAR(1) = nu
   PAR(2) = beta
   PAR(3) = gamma
   PAR(4) = r
   PAR(5) = a3
   PAR(6) = b3

   x = 0.d0
   y = 0.d0
   z = 0.d0

   U(1) = x
   U(2) = y
   U(3) = z

 END SUBROUTINE STPNT
!---------------------------------------------------------------------- 

 SUBROUTINE PVLS(NDIM,U,PAR)
!---------- ----

 IMPLICIT NONE
 INTEGER, INTENT(IN) :: NDIM
 DOUBLE PRECISION, INTENT(IN) :: U(NDIM)
 DOUBLE PRECISION, INTENT(INOUT) :: PAR(*)
 DOUBLE PRECISION, EXTERNAL :: GETP
 DOUBLE PRECISION normx, normy, normz

! Also put the L2-norm in PAR(10):
   normx = GETP('NRM',1,U)
   normy = GETP('NRM',2,U)
   normz = GETP('NRM',3,U)
   
  PAR(10) = DSQRT(normx**2 + normy**2 + normz**2)

 END SUBROUTINE PVLS

!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
 SUBROUTINE BCND 
 END SUBROUTINE BCND

 SUBROUTINE ICND
 END SUBROUTINE ICND

 SUBROUTINE FOPT 
 END SUBROUTINE FOPT
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
